// HelixFragmentationSystems.h is a part of the PYTHIA event generator, adapted by S.Todorova
// Copyright (C) 2012 Torbjorn Sjostrand.
// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// This file contains auxiliary classes in the helix fragmentation process.
// HelixStringSystem contains all the StringRegions of the colour singlet.
// HadronSystem saves the parton content of a hadron, or string piece. 

#ifndef Pythia8_HelixFragmentationSystems_H
#define Pythia8_HelixFragmentationSystems_H

#include "FragmentationSystems.h"

namespace Pythia8 {
  
//==========================================================================

// The HelixStringSystem class contains the complete set of all string regions.
// Only to be used inside HelixStringFragmentation, so no private members.

class HelixStringSystem {

public:

  // Constructor. 
  HelixStringSystem() {}

  // Set up system from parton list.
  void setUp(vector<int>& iSys, Event& event);

  // Calculate string region from (iPos, iNeg) pair.
  int iReg( int iPos, int iNeg) const 
    {return (iPos * (indxReg - iPos)) / 2 + iNeg;}

  // Reference to string region specified by (iPos, iNeg) pair.
  StringRegion& region(int iPos, int iNeg) {return system[iReg(iPos, iNeg)];} 

  // Reference to low string region specified either by iPos or iNeg.
  StringRegion& regionLowPos(int iPos) {
    return system[iReg(iPos, iMax - iPos)]; } 
  StringRegion& regionLowNeg(int iNeg) {
    return system[iReg(iMax - iNeg, iNeg)]; } 

  // Main content: a vector with all the string regions of the system. 
  vector<StringRegion> system;

  // parton fractions - to be used in fragmentation
  vector<std::pair<double, double> > fractions;
  vector<std::pair<double, double> > fractionsTemp;
  void resetFractions();  
  void saveFractions();  
  void restoreFractions();  

  // Other data members.
  int    sizePartons, sizeStrings, sizeRegions, indxReg, iMax; 
  double mJoin, m2Join;
  double phaseTot, phasePos, phaseNeg, chirality;
  bool isGluonLoop;

};
 
//==========================================================================
class HadronSystem {

public:

  // Constructor. 
  HadronSystem() {};

  // Constants: could only be changed in the code itself.
  static const double TINY;

  //  
  // void setFractions(vector<pair<int,double> >& fPos, vector<pair<int,double> >& fNeg) ;
  void setEnds(int iPPos,int iPNeg, double xPos, double yPos,  
               int iNPos,int iNNeg, double xNeg, double yNeg);
  void setPhases( double phPos, double phNeg ) ;
  void setFlavours( int ifPos, int ifNeg ) ;

  Vec4 getMomentum(HelixStringSystem& );
  Vec4 getMomentumPos(HelixStringSystem& );
  Vec4 getMomentumNeg(HelixStringSystem& );
 
  // Data members.
  vector<pair<int,double> > frPos; 
  vector<pair<int,double> > frNeg; 
  int iPP, iPN, iNP, iNN, flavPos, flavNeg;
  double xP, yP, xN, yN, phasePos, phaseNeg; 

};


//==========================================================================

} // end namespace Pythia8

#endif // Pythia8_HelixFragmentationSystems_H
