// HelixFragmentationSystems.cc is a part of the PYTHIA event generatoradapted by S.Todorova.
// Copyright (C) 2012 Torbjorn Sjostrand.
// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// Function definitions (not found in the header) for the
// ColConfig, StringRegion and StringSystem classes.

#include "HelixFragmentationSystems.h"

namespace Pythia8 {


// The HelixStringSystem class.

//--------------------------------------------------------------------------

// Set up system from parton list. 

void HelixStringSystem::setUp(vector<int>& iSys, Event& event) {

  // Figure out how big the system is. (Closed gluon loops?)
  sizePartons = iSys.size();
  sizeStrings = sizePartons - 1;
  sizeRegions = (sizeStrings * (sizeStrings + 1)) / 2;
  indxReg = 2 * sizeStrings + 1;
  iMax = sizeStrings - 1; 

  // ST gluon loops
  isGluonLoop = event[ iSys[0] ].isGluon() ? true : false;

  // Reserve space for the required number of regions.
  system.clear();
  system.resize(sizeRegions);

  fractions.clear(); 
  fractions.resize(sizeStrings);
  resetFractions(); 
  fractionsTemp.resize(sizeStrings);

  phaseTot=0.; 

  // Set up the lowest-lying regions.
  for (int i = 0; i < sizeStrings; ++i) {
    Vec4 p1 = event[ iSys[i] ].p();
    if ( event[ iSys[i] ].isGluon() ) p1 *= 0.5;
    Vec4 p2 = event[ iSys[i+1] ].p();
    if ( event[ iSys[i+1] ].isGluon() ) p2 *= 0.5;
    system[ iReg(i, iMax - i) ].setUp( p1, p2, false);

    if (i!=0 || !event[ iSys[i] ].isGluon() ) {
      phaseTot += sqrt(system[iReg(i,iMax-i)].w2); 
    }

  }

}

//==========================================================================

  void HelixStringSystem::resetFractions() {
    
    for (unsigned int i=0; i<fractions.size(); i++) {
      fractions[i].first=1.;
      fractions[i].second=1.;
    }
  }

//==========================================================================

  void HelixStringSystem::saveFractions() {
    
    for (unsigned int i=0; i<fractions.size(); i++) {
      fractionsTemp[i].first=fractions[i].first;
      fractionsTemp[i].second=fractions[i].second;
    }
  }

//==========================================================================

  void HelixStringSystem::restoreFractions() {
    
    for (unsigned int i=0; i<fractions.size(); i++) {
      fractions[i].first=fractionsTemp[i].first;
      fractions[i].second=fractionsTemp[i].second;
    }
  }


//========================================================================
// The HadronSystem class

  void HadronSystem::setEnds(int iPPos,int iPNeg, double xPos, double yPos,  
			     int iNPos,int iNNeg, double xNeg, double yNeg) {
    iPP = iPPos;
    iPN = iPNeg;
    iNP = iNPos;
    iNN = iNNeg;
    xP = xPos;
    yP = yPos;
    xN = xNeg;
    yN = yNeg;     
  }

  void HadronSystem::setPhases(double phPos,  double phNeg) {
    phasePos = phPos;
    phaseNeg = phNeg; 
  }

  void HadronSystem::setFlavours( int ifPos, int ifNeg ) {
    flavPos =  ifPos;
    flavNeg =  ifNeg;
  }

  Vec4 HadronSystem::getMomentum(HelixStringSystem& sys ) {

    Vec4 pTot;
 
    for (unsigned int i=0; i<frPos.size(); i++)  {
      pTot += frPos[i].second*sys.regionLowPos(frPos[i].first).pPos;
    }
    for (unsigned int i=0; i<frNeg.size(); i++)  {
      pTot += frNeg[i].second*sys.regionLowNeg(frNeg[i].first).pNeg;
    }

   // add pt
    StringRegion regp;
    regp.setUp(sys.regionLowPos(iPP).pPos,
	       sys.regionLowNeg(iPN).pNeg, true);
    pTot += regp.pHad(0.,0.,xP,yP);
    StringRegion regn;
    regn.setUp(sys.regionLowPos(iNP).pPos,
	       sys.regionLowNeg(iNN).pNeg, true);
    pTot += regn.pHad(0.,0.,xN,yN);

    return pTot;
  }

  Vec4 HadronSystem::getMomentumPos(HelixStringSystem& sys ) {

    Vec4 pTot;
 
    for (unsigned int i=0; i<frPos.size(); i++)  {
      pTot += frPos[i].second*sys.regionLowPos(frPos[i].first).pPos;
    }

    return pTot;
  }

  Vec4 HadronSystem::getMomentumNeg(HelixStringSystem& sys ) {

    Vec4 pTot;
 
    for (unsigned int i=0; i<frNeg.size(); i++)  {
      pTot += frNeg[i].second*sys.regionLowNeg(frNeg[i].first).pNeg;
    }

    return pTot;
  }
 
//==========================================================================
 

} // end namespace Pythia8
